import json, sqlite_utils
from rdf_to_sqlite.mappings import rdf_to_sqlite, sqlite_to_rdf
from sqlite_utils import Database
from rdflib import Graph
from pathlib import Path

TEST_RDF = """
{
    "@context": "https://schema.org/docs/jsonldcontext.jsonld",
    "@graph": [
        {
            "id": "https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2676",
            "text": "(The Travellers Companion Series, No. 76)",
            "type": "Comment"
        },
        {
            "author": { "id": "http://www.wikidata.org/entity/Q188176" },
            "comment": [
                { "id": "https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2676" },
                { "id": "https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2677" },
                { "id": "https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2678" },
                { "id": "https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2679" },
                { "id": "https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2680" },
                { "id": "https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2681"}
            ],
            "id": "https://wsburroughs.link/Book/abr40-a2a",
            "name": "Naked Lunch",
            "type": "Book"
        },
        {
            "id": "http://www.wikidata.org/entity/Q188176",
            "type": "Person"
        },
        {
            "id": "https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2677",
            "text": "Dustjacket designed by Burroughs.",
            "type": "Comment"
        },
        {
            "id": "https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2680",
            "text": "2nd printing [not to be confused with the “second issue” of the first printing (above)] issued without  dustjacket or decorative border around title page, and with price (“18 francs”) printed on back cover.",
            "type": "Comment"
        },
        {
            "id": "https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2679",
            "text": "1st printing comprised two “issues:”  1st issue: Price (“Francs: 1,500”) printed in lower right corner of back cover. 2nd issue: Price (“New Price NF 18”) stamped over old price in lower right corner of back cover, following the revaluation of the franc in January 1960.",
            "type": "Comment"
        },
        {
            "id": "https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2678",
            "text": "First copies printed issued without dustjacket. [M&M]",
            "type": "Comment"
        },
        {
            "id": "https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2681",
            "text": "3rd printing (1965) issued without dustjacket or decorative border around title page, and with price (“Francs: h18”) printed on back cover.",
            "type": "Comment"
        }
    ]
}
"""

EXPECTED_COMMENT_ROWS = [
 {'id': 'https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2678',
  'text': 'First copies printed issued without dustjacket. [M&M]'},
 {'id': 'https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2677',
  'text': 'Dustjacket designed by Burroughs.'},
 {'id': 'https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2681',
  'text': '3rd printing (1965) issued without dustjacket or decorative border around title page, and with price (“Francs: h18”) printed on back cover.'},
 {'id': 'https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2679',
  'text': '1st printing comprised two “issues:”  1st issue: Price (“Francs: 1,500”) printed in lower right corner of back cover. 2nd issue: Price (“New Price NF 18”) stamped over old price in lower right corner of back cover, following the revaluation of the franc in January 1960.'},
 {'id': 'https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2676',
  'text': '(The Travellers Companion Series, No. 76)'},
 {'id': 'https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2680',
  'text': '2nd printing [not to be confused with the “second issue” of the first printing (above)] issued without  dustjacket or decorative border around title page, and with price (“18 francs”) printed on back cover.'}
]

EXPECTED_BOOK_COMMENT_ROWS = [
 {'subject': 'https://wsburroughs.link/Book/abr40-a2a',
  'predicate': 'comment',
  'object': 'https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2680'},
 {'subject': 'https://wsburroughs.link/Book/abr40-a2a',
  'predicate': 'comment',
  'object': 'https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2678'},
 {'subject': 'https://wsburroughs.link/Book/abr40-a2a',
  'predicate': 'comment',
  'object': 'https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2676'},
 {'subject': 'https://wsburroughs.link/Book/abr40-a2a',
  'predicate': 'comment',
  'object': 'https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2681'},
 {'subject': 'https://wsburroughs.link/Book/abr40-a2a',
  'predicate': 'comment',
  'object': 'https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2677'},
 {'subject': 'https://wsburroughs.link/Book/abr40-a2a',
  'predicate': 'comment',
  'object': 'https://wsburroughs.link/Comment/n002ffd6182d84355b9967f0665f44e14b2679'}
]

def test_mapping_roundtrip(tmpdir):
    db_path = tmpdir / "test_mappings.db"
    rdf_path = tmpdir / "test_mappings.jsonld"
    Path(rdf_path).write_text(TEST_RDF)
    db = Database(str(db_path))
    graph_in = Graph().parse(str(rdf_path), format='json-ld')
    context = "https://schema.org/docs/jsonldcontext.jsonld"
    rdf_to_sqlite(graph_in, context, db)
    graph_out = sqlite_to_rdf(db, context)
    assert len(graph_in) == len(graph_out)



import json
from rdflib import Graph, URIRef, BNode
from rdflib.namespace import RDF
from rdflib.plugins.shared.jsonld.context import Context
from sqlite_utils import Database

def rdf_to_sqlite(graph, context, db):
    ctxt = Context(source=context)
    tables = set([ o for o in graph.objects(predicate=RDF.type)])
    properties = set([ p for p in graph.predicates() ])
    # We assume that any property with only IRIs or blank nodes as objects is a
    # reference property. RDF that mixes literals with IRIs or blank nodes in
    # the range of a given property will end up with that property treated
    # as a literal property, which will result in funky literal property values.
    reference_properties = set([ p for p in properties if all([ isinstance(o, URIRef) or isinstance(o, BNode) for o in graph.objects(predicate=p)])])
    literal_properties = properties - reference_properties
    instance_triples = {}
    reference_triples = {}
    for table in tables:
        instance_triples[table] = Graph()
        reference_triples[table] = {}
        instances = graph.subjects(predicate=RDF.type, object=table)
        for instance in instances:
            for s, p, o in graph.triples((instance, None, None)):
                if p in literal_properties:
                    instance_triples[table].add((s, p, o))
                else:
                    if f'{p}' not in reference_triples[table]:
                        reference_triples[table][f'{p}'] = Graph()
                    reference_triples[table][f'{p}'].add((s, p, o))
    for table in tables:
        table_name = ctxt.to_symbol(table)
        instance_triples_jsonld = json.loads(instance_triples[table].serialize(format='json-ld', context=context))
        # Bug: need to address literal datatypes f.g. {"@type": "xsd:date", "@value": "2020-06-22"}
        if "@graph" in instance_triples_jsonld:
            rows = instance_triples_jsonld["@graph"]
        else:
            # Bug: need to remove @context from singleton JSON objects, otherwise
            # it is treated as a column/value
            rows = [ instance_triples_jsonld ]
        db[table_name].insert_all(rows, pk='id', alter=True)
    for table in tables:
        for r in reference_triples[table]:
            table_name = f'{ctxt.to_symbol(table)}_{ctxt.to_symbol(r)}'
            rows = []
            for s, p, o in reference_triples[table][r].triples((None, None, None)) :
                if isinstance(s, BNode):
                    subj = s.n3()
                else:
                    subj = ctxt.to_symbol(s)
                prop = ctxt.to_symbol(p)
                if isinstance(o, BNode):
                    obj = o.n3()
                else:
                    obj = ctxt.to_symbol(o)
                # Issue: we could save some space by removing the predicate column,
                # but this will complicate recovery of predicate CURIEs/IRIs
                # during inverse mapping (if and when we add that capability.)
                row = { 'subject': subj, 'predicate': prop, 'object': obj }
                rows.append(row)
            db[table_name].insert_all(rows, pk=('subject', 'object'))

def sqlite_to_rdf(db, context):
    ctxt = Context(source=context)
    objects = []
    for table in db.tables:
        if len(table.pks) < 2:
            for record in table.rows:
                for key in record:
                    if record[key] and record[key][0] == "{" and record[key][-1] == "}":
                        record[key] = json.loads(record[key])
                objects.append(record)
        elif len(table.pks) == 2:
            objects +=  [ { "@id": ctxt.expand(record['subject']), record['predicate']: { "@id": ctxt.expand(record['object']) } } for record in table.rows ]
    return Graph().parse(data=json.dumps(objects), context=context, format='json-ld')



import click
from rdflib import Graph
from sqlite_utils import Database
from .mappings import rdf_to_sqlite

@click.command()
@click.version_option()
@click.argument("db_path", type=click.Path(file_okay=True, dir_okay=False, allow_dash=False))
@click.argument("rdf_file", type=click.Path(file_okay=True, dir_okay=False, allow_dash=False))
@click.option("--context", type=click.STRING, help="URL or file containing a JSON-LD context")
@click.option("--format", required=True, type=click.Choice([
    'html', 'n3', 'nquads', 'nt', 'rdfa',
    'rdfa1.0', 'rdfa1.1', 'trix',
    'turtle', 'xml', 'json-ld'], case_sensitive=False), help='RDF serialization format')
def cli(db_path, rdf_file, context, format):
    "Convert an RDF file to SQLite"
    db = Database(db_path)
    graph = Graph().parse(rdf_file, format=format)
    rdf_to_sqlite(graph, context, db)
    click.echo(f'Generated {db_path} ({sum([ table.count for table in db.tables ])} rows in {len(db.tables)} tables)')

if __name__ == '__main__':
    cli()



from setuptools import setup, find_packages
import io
import os

VERSION = "0.2"

def get_long_description():
    with io.open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"),
        encoding="utf8",
    ) as fp:
        return fp.read()

setup(
    name="rdf-to-sqlite",
    description="Utility for converting an RDF file to SQLite",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Bradley P. Allen",
    version=VERSION,
    license="MIT License",
    packages=find_packages(),
    install_requires=["click>=8.0.1", "rdflib>=6.0.0", "sqlite-utils>=3.12"],
    setup_requires=["pytest-runner"],
    extras_require={"test": ["pytest"]},
    entry_points="""
        [console_scripts]
        rdf-to-sqlite=rdf_to_sqlite.cli:cli
    """,
    tests_require=["rdf-to-sqlite[test]"],
    url="https://github.com/bradleypallen/rdf-to-sqlite",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)
